    #######################################################################
    # Module: EGRRext
    # Name  : EG (exported member)
    #
    # Input:
    #     EM0 - the list [M, l, t] where M is the matrix n by m*n, 
    #           an explicit matrix of <A_{l}|A_{l-1}|...|A_t> of
    #           the operator A_l*E^l+A_{l-1}*E^{l-1}+...+A_t*E^t, 
    #           where E - is the shift operator: Ey(x)->y(x+1),
    #           l and t are the corresponding leading and trailing orders;
    #           if t=0 then just M may be given instead of the list [M,l,t] 
    #     x - variable
    #     lt - one of 'lead' or 'trail' ('lead' is by default), parameter 
    #           determining which of 
    #           the matrices (leading A_{m-1} or trailing A_0) should be revealed
    #     U (optional) - name to be assigned to the unimodular matrix U calculated 
    #           by the extended version of RR
    #           (the version is used if the name is specified)
    #     U0 (optional) - initial value of U if the transformation is performed  
    #           in a row (used in IsUnimodular)
    #
    # Output:
    #     EM, AreIndependent:
    #     [EM,l,t] - EM is the explicit matrix after the transformation by 
    #           EG algorithm, with the leading (resp. trailing) matrix 
    #           being non-siglular if the rows of the given operator are 
    #           independent, l and t are the correspondingleading and trailing 
    #           orders of EM
    #     AreIndependent - true or false, determines whether the rows
    #           of the given operator are independent
    #
    #######################################################################

    EG := proc(EM0, x, lt:='lead')
        local
            EM, rd, cd, n, m, m1, m2, l, t, RM, ns, p, dummy, i, j, alphai, shifts, r, d, needU, U, U0, tl, cdU, dummyU, lU, tU, mU, lU0, tU0;

        needU :=`if`((nargs>=4) and type(args[4], name), true, false);
        U0 :=`if`((nargs>=5) and type(args[5], list), args[5], NULL);

        if type(EM0, list) then
            EM := copy(EM0[1]);
        else
            EM := copy(EM0);
        end if;
        rd := LinearAlgebra:-RowDimension(EM);
        cd := LinearAlgebra:-ColumnDimension(EM);
        m := trunc(cd/rd);
        n := trunc(cd/m);
        if n<>rd then
            error("For the unimodularity testing, the given operator matrix (i.e. the explicit matrix blocks) should be square.");  
        end if;
        if type(EM0, list) then
            l := EM0[2];
            t := EM0[3];
        else
            l := m-1;
            t := 0;
        end if;

        if lt='lead' then
            tl := 'trail';
        else
            tl := 'lead';
        end if;

        if needU then
            if n<>rd then
                error("For the extenden version of EG, the given operator matrix (i.e. the explicit matrix blocks) should be square.");  
            end if;
            if U0<>NULL then
                lU0 := U0[2];
                tU0 := U0[3];
                cdU := rd*rd*(m-1)+rd*(lU0-tU0+1);
                U := Matrix(rd,cdU,0);
                if lt = 'lead' then            
                    U[1..rd, cdU-LinearAlgebra:-ColumnDimension(U0[1])+1..cdU] := U0[1];
                    lU := rd*(m-1)+lU0;
                    tU := tU0;
                else
                    U[1..rd, 1..LinearAlgebra:-ColumnDimension(U0[1])] := U0[1];
                    lU := lU0;
                    tU := tU0-rd*(m-1);
                end if;                    
            else
                cdU := rd*rd*(m-1)+rd;
                U := Matrix(rd,cdU,0);
                if lt='lead' then
                    for i to rd do
                        U[i, cdU-rd+i] := 1;
                    end do;
                    lU := rd*(m-1);
                    tU := 0;
                else
                    for i to rd do
                        U[i, i] := 1;
                    end do;
                    lU := 0;
                    tU := -rd*(m-1);
                end if;
            end if;
            mU := lU-tU;
        end if;
        
        shifts := 0;
        while shifts<cd+1 do
            if lt='lead' then
                RM := LinearAlgebra:-SubMatrix(EM, 1 .. rd, 1 .. n);
            else
                RM := LinearAlgebra:-SubMatrix(EM, 1 .. rd, cd-n+1 .. cd);
            end if;
            ns := LinearAlgebra:-NullSpace(LinearAlgebra:-Transpose(RM));
            if nops(ns)=0 then
                break;
            end if;
            ns := SolveTools:-SortByComplexity(convert(ns,list));
            p := LinearAlgebra:-Transpose(ns[1]);
            i := 0; 
            alphai := -infinity;
            for j to rd do 
                if p[j]<>0 then
                    d := alpha(EM[j], n, cd, tl);
                    if (d>alphai) then# or ((d=alphai) and (measure(p[j], x)<measure(p[i], x))) then
                        i := j;
                        alphai := d;
                    end if;
                end if; 
            end do;
            p := map(z->dualnormal(z/p[i]),p);
            dummy := LinearAlgebra:-VectorMatrixMultiply(p, EM);
            d := alpha(dummy, n, cd, lt);
            if d=-1 then
                shifts := cd+1;
                break
            end if;
            r := m-(d+1);
            shifts := shifts + r;
            EM[i] := ShiftRow(eval(dummy), r, x, n, cd, lt);
            userinfo(3, EGRRext, `reduction and shift in the row`, i, `its length is`, length(EM[i]), `its maximal sum of degrees of numerator and denominator of entries is`, 
                                    max(map(z->degree(numer(z),x)+degree(denom(z),x),convert(EM[i],list))));
            if needU then
               dummyU := LinearAlgebra:-VectorMatrixMultiply(p, U);
               U[i] := ShiftRow(eval(dummyU), r, x, n, cdU, lt);
            end if;
        end do;
        if needU then        
            dummy := [seq(alpha(U[i], n, cdU, 'lead'), i = 1..rd)];
            lU0 := max(dummy);
            dummy := [seq(alpha(U[i], n, cdU, 'trail'), i = 1..rd)];
            tU0 := max(dummy);        
            U := LinearAlgebra:-Map(dualnormal,U[1..rd, cdU-(lU0+1)*rd+1..(tU0+1)*rd]); 
            lU := lU-(mU-lU0);
            tU := tU+(mU-tU0);
            assign(args[4], [U,lU,tU]);
        end if;
        eval(EM), evalb(shifts<cd+1);
    end proc:

    #######################################################################

