    #######################################################################
    # Module: EGRRext
    # Name  : IsUnimodular (exported member)
    #
    # Input:
    #     EM0 - the list [M, l, t] where M is the matrix n by m*n, 
    #           an explicit matrix of <A_{l}|A_{l-1}|...|A_t> of
    #           the operator A_l*E^l+A_{l-1}*E^{l-1}+...+A_t*E^t, 
    #           where E - is the shift operator: Ey(x)->y(x+1),
    #           l and t are the corresponding leading and trailing orders;
    #           if t=0 then just M may be given instead of the list [M,l,t] 
    #     x - variable
    #     Iname (optional) - name to be assigned to the inverse of 
    #           the given operator is it is unimodular (specified in
    #           the same form as EM0).
    #
    # Output:
    #     true if the given operator is unimodular,
    #       false otherwise (for the differential and difference case,
    #       q-difference cases is not supported yet)
    #
    #######################################################################

    IsUnimodular := proc(EM0, x, Iname::name)
        local
            EM, U, M, alphas, needI, i, k, l, m, s, t, rd, cd, cdEM, cdU, cdM, n, IsIndependent, RM, RMord;

        if type(EM0, list) then
            EM := copy(EM0[1]);
        else
            EM := copy(EM0);
        end if;
        rd := LinearAlgebra:-RowDimension(EM);
        cd := LinearAlgebra:-ColumnDimension(EM);
        m := trunc(cd/rd);
        n := trunc(cd/m);
        if n<>rd then
            error("For the unimodularity testing, the given operator matrix (i.e. the explicit matrix blocks) should be square.");  
        end if;
        if type(EM0, list) then
            l := EM0[2];
            t := EM0[3];
        else
            l := m-1;
            t := 0;
        end if;
        needI :=`if`(nargs=3, true, false);
        if needI then
            EM, IsIndependent := EG([EM, l, t], x, 'trail', U);
            if not IsIndependent then
                return(false);
            end if;
            EM, IsIndependent := EG([EM, l, t], x, 'lead', 'U', U);
            cdEM := LinearAlgebra:-ColumnDimension(EM);
            cdU := LinearAlgebra:-ColumnDimension(U[1]);
            alphas := [[seq(alpha(EM[i], n, cdEM, 'lead'), i = 1..rd)],map(z->(m-1)-z,[seq(alpha(EM[i], n, cdEM, 'trail'), i = 1..rd)])];
            if not IsIndependent or (add(alphas[1]-alphas[2]))<>0 then
                return(false);
            end if;
            RM := Matrix([seq([EM[i,(m-1-alphas[2][i])*n+1..(m-1-alphas[2][i])*n+n]], i = 1..rd)]);
            RMord := max(alphas[1]);
            M := LinearAlgebra:-MatrixMatrixMultiply(LinearAlgebra:-MatrixInverse(RM), U[1]);
            M := eval(M, x=x-RMord-t);
            cdM := LinearAlgebra:-ColumnDimension(M);
            alphas := [seq(alpha(M[i], n, cdM, 'lead'), i = 1..rd)];
            assign(Iname, [M, U[3]+max(alphas)-RMord-t, U[3]-RMord-t]);
            return(true);
        else
            EM, IsIndependent := EG([EM, l, t], x, 'trail');
            if not IsIndependent then
                return(false);
            end if;
            EM, IsIndependent := EG([EM, l, t], x, 'lead');
            cdEM := LinearAlgebra:-ColumnDimension(EM);
            alphas := [[seq(alpha(EM[i], n, cdEM, 'lead'), i = 1..rd)],map(z->(m-1)-z,[seq(alpha(EM[i], n, cdEM, 'trail'), i = 1..rd)])];
            if not IsIndependent or (add(alphas[1]-alphas[2]))<>0 then
                return(false);
            end if;
            return(true);
        end if;            
    end proc:

    #######################################################################

